@@ -903,6 +903,69 @@ TEST_CASE("quantiles sketch", "[quantiles_sketch]") {
     }
   }
 
+  SECTION("Type converting copy constructor") {
+    const uint16_t k = 8;
+    const int n = 403;
+    quantiles_sketch<double> sk_double(k);
+    
+    quantiles_sketch<float> sk_float(k, sk_double.get_allocator());
+    REQUIRE(sk_float.is_empty());
+    
+    for (int i = 0; i < n; ++i) sk_double.update(i + .01);
+
+    quantiles_sketch<int> sk_int(sk_double);
+    REQUIRE(sk_double.get_n() == sk_int.get_n());
+    REQUIRE(sk_double.get_k() == sk_int.get_k());
+    REQUIRE(sk_double.get_num_retained() == sk_int.get_num_retained());
+
+    auto sv_double = sk_double.get_sorted_view(false);
+    std::vector<std::pair<double, uint64_t>> vec_double(sv_double.begin(), sv_double.end()); 
+
+    auto sv_int = sk_int.get_sorted_view(false);
+    std::vector<std::pair<int, uint64_t>> vec_int(sv_int.begin(), sv_int.end()); 
+
+    REQUIRE(vec_double.size() == vec_int.size());
+    
+    for (size_t i = 0; i < vec_int.size(); ++i) {
+      // known truncation with conversion so approximate result
+      REQUIRE(vec_double[i].first == Approx(vec_int[i].first).margin(0.1));
+      // exact equality for weights
+      REQUIRE(vec_double[i].second == vec_int[i].second);
+    }
+  }
+
+  class A {
+    int val;
+  public:
+    A(int val): val(val) {}
+    int get_val() const { return val; }
+  };
+
+  struct less_A {
+    bool operator()(const A& a1, const A& a2) const { return a1.get_val() < a2.get_val(); }
+  };
+
+  class B {
+    int val;
+  public:
+    explicit B(const A& a): val(a.get_val()) {}
+    int get_val() const { return val; }
+  };
+
+  struct less_B {
+    bool operator()(const B& b1, const B& b2) const { return b1.get_val() < b2.get_val(); }
+  };
+
+  SECTION("type conversion: custom types") {
+    quantiles_sketch<A, less_A> sa;
+    sa.update(1);
+    sa.update(2);
+    sa.update(3);
+
+    quantiles_sketch<B, less_B> sb(sa);
+    REQUIRE(sb.get_n() == 3);
+  }
+
   // cleanup
   if (test_allocator_total_bytes != 0) {
     REQUIRE(test_allocator_total_bytes == 0);
